==> ducene-0.0.3.tar.gz: lib/src/index_writer.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class IndexWriter {

  final IndexDirectory _directory;

  IndexWriter(this._directory);

  void write(Iterable<Document> documents) {
    int docId = 0;
    var indexes = <String,InvertedIndexBuilder>{};
    var data = <String,StoredDataBuilder>{};
    for (Document doc in documents) {
      _updateDocument(docId, doc, indexes, data);
      docId++;
    }
    _storeIndexes(indexes);
    _storeData(data);
  }

  void _updateDocument(int docId, Document document,
      Map<String,InvertedIndexBuilder> indexes,
      Map<String,StoredDataBuilder> data) {
    for (Field field in document.fields) {
      String name = field.name;
      List<String> text = field.text;
      var tokensIte = new Iterable.generate(text.length,
            (i) => field.analyzer.getTokens(text[i]));
      Iterable<String> tokens = concat(tokensIte);
      if (!indexes.containsKey(name)) {
        indexes[name] = new InvertedIndexBuilder().append(tokens, docId);
      } else {
        indexes[name].append(tokens, docId);
      }
      if (!data.containsKey(name)) {
        data[name] = new StoredDataBuilder().append(docId, field.text);
      } else {
        data[name].append(docId, field.text);
      }
    }
  }

  void _storeIndexes(Map<String,InvertedIndexBuilder> indexes) {
    for (String name in indexes.keys) {
      _directory.outputIndex(name, indexes[name].build());
    }
  }

  void _storeData(Map<String,StoredDataBuilder> data) {
    for (String name in data.keys) {
       _directory.outputData(name, data[name].build());
    }
  }

}

class InvertedIndexBuilder {

  ListMultimap<String,int> _invertedIndex = new ListMultimap();

  InvertedIndexBuilder append(Iterable<String> tokens, int docId) {
    for (String token in tokens) {
      if (!_invertedIndex[token].contains(docId)) {
        _invertedIndex.add(token, docId);
      }
    }
    return this;
  }

  ListMultimap<String,int> build() {
    ListMultimap<String,int> ret = new ListMultimap();
    List<String> terms = _invertedIndex.keys.toList();
    terms.sort();
    for (String term in terms) {
      ret.addValues(term, _invertedIndex[term]);
    }
    return ret;
  }
}

class StoredDataBuilder {

  ListMultimap<int,String> _storedData = new ListMultimap();

  StoredDataBuilder append(int docId, List<String> text) {
    _storedData.addValues(docId, text);
    return this;
  }

  ListMultimap<int,String> build() {
    return _storedData;
  }

}

==> ducene-0.0.5.tar.gz: lib/src/index_writer.dart <==
// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class IndexWriter {

  final IndexDirectory _directory;

  IndexWriter(this._directory);

  void write(Iterable<Document> documents) {
    int docId = 0;
    var indexes = <String,InvertedIndexBuilder>{};
    var data = <String,StoredDataBuilder>{};
    for (Document doc in documents) {
      _updateDocument(docId, doc, indexes, data);
      docId++;
    }
    _storeIndexes(indexes);
    _storeData(data);
  }

  void _updateDocument(int docId, Document document,
      Map<String,InvertedIndexBuilder> indexes,
      Map<String,StoredDataBuilder> data) {
    for (Field field in document.fields) {
      String name = field.name;
      List<String> text = field.text;
      var tokensIte = new Iterable.generate(text.length,
            (i) => field.analyzer.getTokens(text[i]));
      Iterable<String> tokens = tokensIte.expand((x) => x);
      if (!indexes.containsKey(name)) {
        indexes[name] = new InvertedIndexBuilder().append(tokens, docId);
      } else {
        indexes[name].append(tokens, docId);
      }
      if (field.stored) {
        if (!data.containsKey(name)) {
          data[name] = new StoredDataBuilder().append(docId, text);
        } else {
          data[name].append(docId, field.text);
        }
      }
    }
  }

  void _storeIndexes(Map<String,InvertedIndexBuilder> indexes) {
    for (String name in indexes.keys) {
      _directory.outputIndex(name, indexes[name].build());
    }
  }

  void _storeData(Map<String,StoredDataBuilder> data) {
    for (String name in data.keys) {
       _directory.outputData(name, data[name].build());
    }
  }

}

class InvertedIndexBuilder {

  var _invertedIndex = <String,List<int>>{};

  InvertedIndexBuilder append(Iterable<String> tokens, int docId) {
    for (String token in tokens) {
      if(!_invertedIndex.containsKey(token)) {
        var list = <int>[]..add(docId);
        _invertedIndex[token] = list;
      } else {
        var list = _invertedIndex[token];
        if (!list.contains(docId)) {
           list.add(docId);
         }
      }
    }
    return this;
  }

  Map<String,List<int>> build() {
    var ret = {};
    List<String> terms = _invertedIndex.keys.toList();
    terms.sort();
    for (String term in terms) {
      ret[term] = _invertedIndex[term];
    }
    return ret;
  }
}

class StoredDataBuilder {

  var _storedData = <int,List<String>>{};

  StoredDataBuilder append(int docId, List<String> text) {
    _storedData[docId] = text;
    return this;
  }

  Map<int,List<String>> build() {
    return _storedData;
  }

}
